;****************************************************************************
; MODULO DE CONTROL - KEYBOARD
;
; Indices de las teclas convencionales:
;   1    ESC                               44   Z
;   2    1                                 45   X
;   3    2                                 46   C
;   4    3                                 47   V
;   5    4                                 48   B
;   6    5                                 49   N
;   7    6                                 50   M
;   8    7                                 51   , <
;   9    8                                 52   . >
;   10   9                                 53   / ?
;   11   0                                 54   RIGHT SHIFT
;   12   - _                               55   *            (KEYPAD)
;   13   = +                               56   LEFT ALT
;   14   BACKSPACE                         57   SPACEBAR
;   15   TAB                               58   CAPSLOCK
;   16   Q                                 59   F1
;   17   W                                 60   F2
;   18   E                                 61   F3
;   19   R                                 62   F4
;   20   T                                 63   F5
;   21   Y                                 64   F6
;   22   U                                 65   F7
;   23   I                                 66   F8
;   24   O                                 67   F9
;   25   P                                 68   F10
;   26   [                                 69   NUMLOCK      (KEYPAD)
;   27   ]                                 70   SCROLL LOCK
;   28   ENTER (RETURN)                    71   7 HOME       (KEYPAD)
;   29   LEFT CONTROL                      72   8 UP         (KEYPAD)
;   30   A                                 73   9 PGUP       (KEYPAD)
;   31   S                                 74   -            (KEYPAD)
;   32   D                                 75   4 LEFT       (KEYPAD)
;   33   F                                 76   5            (KEYPAD)
;   34   G                                 77   6 RIGHT      (KEYPAD)
;   35   H                                 78   +            (KEYPAD)
;   36   J                                 79   1 END        (KEYPAD)
;   37   K                                 80   2 DOWN       (KEYPAD)
;   38   L                                 81   3 PGDN       (KEYPAD)
;   39   ; :                               82   0 INSERT     (KEYPAD)
;   40   ' "                               83   . DEL        (KEYPAD)
;   41   ` ~                               87   F11
;   42   LEFT SHIFT                        88   F12
;
; Indices de las teclas extendidas:
;   156  ENTER            (KEYPAD)         203  LEFT
;   157  RIGHT CONTROL                     205  RIGHT
;   170  PRINT SCREEN                      207  END
;   181  /                (KEYPAD)         208  DOWN
;   183  PRINT SCREEN                      209  PAGE DOWN
;   184  RIGHT ALT                         210  INSERT
;   199  HOME                              211  DELETE
;   200  UP                                239  MACRO
;   201  PAGE UP
;****************************************************************************

;****************************************************************************
; Define los valores.
;****************************************************************************

PUBLIC MCK_INICIA_INTERRUPCION
PUBLIC MCK_FINALIZA_INTERRUPCION

PUBLIC MCK_ULTIMA_TECLA
PUBLIC MCK_TECLAS

INTERRUPCION_SERVICIO     = 021H
FUNCION_PONE_INTERRUPCION = 025H
FUNCION_TOMA_INTERRUPCION = 035H
INTERRUPCION_TECLADO      = 009H

PUERTO_TECLADO = 060H

PUERTO_CONTROL_INTERRUPCION = 20H
RECONOCIMIENTO_INTERRUPCION = 20H

CAMINO SEGMENT PUBLIC
  ASSUME CS:CAMINO,DS:CAMINO,SS:CAMINO,ES:CAMINO

  DB 'MCK 1998 Marco Antonio Alvarado'

;****************************************************************************
; Inicia la nueva interrupcin del teclado. Guarda la direccin de la
; anterior.
;
; Llama:
;   INTERRUPCION_SERVICIO
;
; Escribe:
;   CAMPO_VIEJA_INTERRUPCION
;   SEGMENTO_VIEJA_INTERRUPCION
;****************************************************************************

  LABEL MCK_INICIA_INTERRUPCION NEAR
  INICIA_INTERRUPCION PROC NEAR
    PUSH AX
    PUSH BX
    PUSH ES
    PUSH DX

    MOV AH,FUNCION_TOMA_INTERRUPCION
    MOV AL,INTERRUPCION_TECLADO
    INT INTERRUPCION_SERVICIO
    MOV CAMPO_VIEJA_INTERRUPCION,BX
    MOV SEGMENTO_VIEJA_INTERRUPCION,ES

    MOV AH,FUNCION_PONE_INTERRUPCION
    MOV AL,INTERRUPCION_TECLADO
    LEA DX,ATIENDE_INTERRUPCION
    INT INTERRUPCION_SERVICIO

    POP DX
    POP ES
    POP BX
    POP AX
    RET
  INICIA_INTERRUPCION ENDP

;****************************************************************************
; Finaliza la vieja interrupcin del teclado. Restable la direccin de la
; interrupcin anterior.
;
; Llama:
;   INTERRUPCION_SERVICIO
;
; Lee:
;   CAMPO_VIEJA_INTERRUPCION
;   SEGMENTO_VIEJA_INTERRUPCION
;****************************************************************************

  LABEL MCK_FINALIZA_INTERRUPCION NEAR
  FINALIZA_INTERRUPCION PROC NEAR
    PUSH AX
    PUSH DX
    PUSH BX
    PUSH DS

    MOV AH,FUNCION_PONE_INTERRUPCION
    MOV AL,INTERRUPCION_TECLADO
    MOV DX,CAMPO_VIEJA_INTERRUPCION
    MOV BX,SEGMENTO_VIEJA_INTERRUPCION
    MOV DS,BX
    INT INTERRUPCION_SERVICIO

    POP DS
    POP BX
    POP DX
    POP AX
    RET
  FINALIZA_INTERRUPCION ENDP

;****************************************************************************
; Atiende la interrupcin del teclado y recibe la tecla.
;
; Lee:
;   TECLA_EXTENDIDA
;
; Escribe:
;   TECLA_EXTENDIDA
;   TECLAS
;****************************************************************************

  ATIENDE_INTERRUPCION PROC FAR
    PUSH AX
    PUSH DS
    PUSH BX

;----------------------------------------------------------------------------
; Establece el segmento de datos de este programa.
    MOV AX,CS
    MOV DS,AX

    IN AL,PUERTO_TECLADO

;----------------------------------------------------------------------------
; Espera una tecla extendida?
    CMP TECLA_EXTENDIDA,0
    JNE AT_ALMACENA_TECLA_EXTENDIDA

;----------------------------------------------------------------------------
; La prxima tecla es extendida?
    CMP AL,224
    JE AT_ESPERA_TECLA_EXTENDIDA

;----------------------------------------------------------------------------
; Almacena la tecla convencional.
    XOR BX,BX
    MOV BL,AL
    AND BL,127
    AND AL,128
    ROL AL,1
    MOV ULTIMA_TECLA,BL
    MOV TECLAS[BX],AL
    JMP AT_FINALIZA

  AT_ESPERA_TECLA_EXTENDIDA:

;----------------------------------------------------------------------------
; Activa la bandera.
    MOV TECLA_EXTENDIDA,1
    JMP AT_FINALIZA

  AT_ALMACENA_TECLA_EXTENDIDA:

;----------------------------------------------------------------------------
; Desactiva la bandera y almacena la tecla extendida.
    MOV TECLA_EXTENDIDA,0
    XOR BX,BX
    MOV BL,AL
    OR BL,128
    AND AL,128
    ROL AL,1
    MOV ULTIMA_TECLA,BL
    MOV TECLAS[BX],AL

  AT_FINALIZA:

;----------------------------------------------------------------------------
; Indica al controlador de interrupciones (8259A MPIC) que reconoce la
; interrupcin.
    MOV AL,RECONOCIMIENTO_INTERRUPCION
    OUT PUERTO_CONTROL_INTERRUPCION,AL

    POP BX
    POP DS
    POP AX
    IRET
  ATIENDE_INTERRUPCION ENDP

;****************************************************************************
; Almacena los valores.
;****************************************************************************

  CAMPO_VIEJA_INTERRUPCION    DW 0
  SEGMENTO_VIEJA_INTERRUPCION DW 0

  TECLA_EXTENDIDA DB 0

  LABEL MCK_ULTIMA_TECLA BYTE
  ULTIMA_TECLA DB 0
  LABEL MCK_TECLAS BYTE
  TECLAS DB 256 DUP (0)

CAMINO ENDS

END