;****************************************************************************
; MODULO DE CONTROL - KEYBOARD
;
; PRUEBA 2 - Recibe las teclas y almacena su estado en un vector.
;****************************************************************************

;****************************************************************************
; Define los valores.
;****************************************************************************

  PUBLIC MCK_INICIA_INTERRUPCION
  PUBLIC MCK_FINALIZA_INTERRUPCION

  PUBLIC MCK_TECLAS

  INTERRUPCION_SERVICIO     = 021H
  FUNCION_PONE_INTERRUPCION = 025H
  FUNCION_TOMA_INTERRUPCION = 035H
  INTERRUPCION_TECLADO      = 009H

  PUERTO_TECLADO = 060H

  PUERTO_CONTROL_INTERRUPCION = 20H
  RECONOCIMIENTO_INTERRUPCION = 20H

CAMINO SEGMENT PUBLIC
  ASSUME CS:CAMINO,DS:CAMINO,SS:CAMINO,ES:CAMINO

;****************************************************************************
; Instala la nueva interrupcin para el teclado.
;
; Llama:
;   INTERRUPCION_SERVICIO
;
; Escribe:
;   CAMPO_VIEJA_INTERRUPCION
;   SEGMENTO_VIEJA_INTERRUPCION
;****************************************************************************

  LABEL MCK_INICIA_INTERRUPCION NEAR
  INICIA_INTERRUPCION PROC NEAR
    PUSH AX
    PUSH BX
    PUSH ES
    PUSH DX

    MOV AH,FUNCION_TOMA_INTERRUPCION
    MOV AL,INTERRUPCION_TECLADO
    INT INTERRUPCION_SERVICIO
    MOV CAMPO_VIEJA_INTERRUPCION,BX
    MOV SEGMENTO_VIEJA_INTERRUPCION,ES

    MOV AH,FUNCION_PONE_INTERRUPCION
    MOV AL,INTERRUPCION_TECLADO
    LEA DX,ATIENDE_TECLADO
    INT INTERRUPCION_SERVICIO

    POP DX
    POP ES
    POP BX
    POP AX
    RET
  INICIA_INTERRUPCION ENDP

;****************************************************************************
; Restaura la vieja interrupcin del teclado.
;
; Llama:
;   INTERRUPCION_SERVICIO
;
; Lee:
;   CAMPO_VIEJA_INTERRUPCION
;   SEGMENTO_VIEJA_INTERRUPCION
;****************************************************************************

  LABEL MCK_FINALIZA_INTERRUPCION NEAR
  FINALIZA_INTERRUPCION PROC NEAR
    PUSH AX
    PUSH DX
    PUSH BX
    PUSH DS

    MOV AH,FUNCION_PONE_INTERRUPCION
    MOV AL,INTERRUPCION_TECLADO
    MOV DX,CAMPO_VIEJA_INTERRUPCION
    MOV BX,SEGMENTO_VIEJA_INTERRUPCION
    MOV DS,BX
    INT INTERRUPCION_SERVICIO

    POP DS
    POP BX
    POP DX
    POP AX
    RET
  FINALIZA_INTERRUPCION ENDP

;****************************************************************************
; Recibe los valores del puerto del teclado.
;
; Lee:
;   TECLA_EXTENDIDA
;
; Escribe:
;   TECLA_EXTENDIDA
;   TECLAS
;****************************************************************************

  ATIENDE_TECLADO PROC FAR
    PUSH AX
    PUSH DS
    PUSH BX

;----------------------------------------------------------------------------
; Establece el segmento de datos de este programa.
    MOV AX,CS
    MOV DS,AX

    IN AL,PUERTO_TECLADO

;----------------------------------------------------------------------------
; Espera una tecla extendida?
    CMP TECLA_EXTENDIDA,0
    JNE AT_ALMACENA_TECLA_EXTENDIDA

;----------------------------------------------------------------------------
; La prxima tecla es extendida?
    CMP AL,224
    JE AT_ESPERA_TECLA_EXTENDIDA

;----------------------------------------------------------------------------
; Almacena la tecla convencional.
    XOR BX,BX
    MOV BL,AL
    AND BL,127
    AND AL,128
    ROL AL,1
    MOV TECLAS[BX],AL
    JMP AT_FINALIZA

  AT_ESPERA_TECLA_EXTENDIDA:

;----------------------------------------------------------------------------
; Activa la bandera.
    MOV TECLA_EXTENDIDA,1
    JMP AT_FINALIZA

  AT_ALMACENA_TECLA_EXTENDIDA:

;----------------------------------------------------------------------------
; Desactiva la bandera y almacena la tecla extendida.
    MOV TECLA_EXTENDIDA,0
    XOR BX,BX
    MOV BL,AL
    OR BL,128
    AND AL,128
    ROL AL,1
    MOV TECLAS[BX],AL

  AT_FINALIZA:

;----------------------------------------------------------------------------
; Indica al controlador de interrupciones (8259A MPIC) que reconoce la
; interrupcin.
    MOV AL,RECONOCIMIENTO_INTERRUPCION
    OUT PUERTO_CONTROL_INTERRUPCION,AL

    POP BX
    POP DS
    POP AX
    IRET
  ATIENDE_TECLADO ENDP

;****************************************************************************
; Almacena los valores.
;****************************************************************************

  CAMPO_VIEJA_INTERRUPCION    DW 0
  SEGMENTO_VIEJA_INTERRUPCION DW 0

  TECLA_EXTENDIDA DB 0

  LABEL MCK_TECLAS BYTE
  TECLAS DB 256 DUP (0)

CAMINO ENDS

END